/**************************************************************************/
/*  camera_linux.h                                                        */
/**************************************************************************/
/*                         This file is part of:                          */
/*                             GODOT ENGINE                               */
/*                        https://godotengine.org                         */
/**************************************************************************/
/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
/*                                                                        */
/* Permission is hereby granted, free of charge, to any person obtaining  */
/* a copy of this software and associated documentation files (the        */
/* "Software"), to deal in the Software without restriction, including    */
/* without limitation the rights to use, copy, modify, merge, publish,    */
/* distribute, sublicense, and/or sell copies of the Software, and to     */
/* permit persons to whom the Software is furnished to do so, subject to  */
/* the following conditions:                                              */
/*                                                                        */
/* The above copyright notice and this permission notice shall be         */
/* included in all copies or substantial portions of the Software.        */
/*                                                                        */
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
/**************************************************************************/

#ifndef CAMERA_LINUX_H
#define CAMERA_LINUX_H

#include "core/os/mutex.h"
#include "core/os/thread.h"
#include "servers/camera_server.h"

class CameraLinux : public CameraServer {
private:
	SafeFlag exit_flag;
	Thread camera_thread;
	Mutex camera_mutex;

	static void camera_thread_func(void *p_camera_linux);

	void _update_devices();
	bool _has_device(const String &p_device_name);
	void _add_device(const String &p_device_name);
	void _remove_device(const String &p_device_name);
	int _open_device(const String &p_device_name);
	bool _is_active(const String &p_device_name);
	bool _is_video_capture_device(int p_file_descriptor);
	bool _can_query_format(int p_file_descriptor, int p_type);

public:
	CameraLinux();
	~CameraLinux();
};

#endif // CAMERA_LINUX_H
